@@ -1,5 +1,5 @@
 /* cat -- concatenate files and print on the standard output.
-   Copyright (C) 88, 90, 91, 1995-2000 Free Software Foundation, Inc.
+   Copyright (C) 88, 90, 91, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,19 +33,18 @@
 #include "system.h"
 #include "closeout.h"
 #include "error.h"
+#include "full-write.h"
 #include "safe-read.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "cat"
 
-#define AUTHORS "Torbjorn Granlund and Richard M. Stallman"
+#define AUTHORS N_ ("Torbjorn Granlund and Richard M. Stallman")
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef max
 #define max(h,i) ((h) > (i) ? (h) : (i))
 
-int full_write ();
-
 /* Name under which this program was invoked.  */
 char *program_name;
 
@@ -156,7 +155,7 @@ simple_cat (
      int bufsize)
 {
   /* Actual number of characters read, and therefore written.  */
-  int n_read;
+  ssize_t n_read;
 
   /* Loop until the end of the file.  */
 
@@ -179,7 +178,7 @@ simple_cat (
 
       /* Write this block out.  */
 
-      if (full_write (STDOUT_FILENO, buf, n_read) < 0)
+      if (full_write (STDOUT_FILENO, buf, n_read) != n_read)
 	error (EXIT_FAILURE, errno, _("write error"));
     }
 }
@@ -226,7 +225,7 @@ cat (
   unsigned char *bpout;
 
   /* Number of characters read by the last read call.  */
-  int n_read;
+  ssize_t n_read;
 
   /* Determines how many consecutive newlines there have been in the
      input.  0 newlines makes NEWLINES -1, 1 newline makes NEWLINES 1,
