@@ -153,9 +153,10 @@ static const char *const charname[33] =
 /* Address base (8, 10 or 16).  */
 static int address_base;
 
-/* The number of octal digits required to represent the largest off_t value.  */
+/* The number of octal digits required to represent the largest
+   address value.  */
 #define MAX_ADDRESS_LENGTH \
-  ((sizeof (off_t) * CHAR_BIT + CHAR_BIT - 1) / 3)
+  ((sizeof (uintmax_t) * CHAR_BIT + CHAR_BIT - 1) / 3)
 
 /* Width of a normal address.  */
 static int address_pad_len;
@@ -173,22 +174,24 @@ static int flag_pseudo_start;
 
 /* The difference between the old-style pseudo starting address and
    the number of bytes to skip.  */
-static off_t pseudo_offset;
+static uintmax_t pseudo_offset;
 
 /* Function that accepts an address and an optional following char,
    and prints the address and char to stdout.  */
-static void (*format_address) PARAMS ((off_t, char));
+static void (*format_address) PARAMS ((uintmax_t, char));
 
 /* The number of input bytes to skip before formatting and writing.  */
-static off_t n_bytes_to_skip = 0;
+static uintmax_t n_bytes_to_skip = 0;
 
-/* When nonzero, MAX_BYTES_TO_FORMAT is the maximum number of bytes
-   to be read and formatted.  Otherwise all input is formatted.  */
+/* When zero, MAX_BYTES_TO_FORMAT and END_OFFSET are ignored, and all
+   input is formatted.  */
 static int limit_bytes_to_format = 0;
 
-/* The maximum number of bytes that will be formatted.  This
-   value is used only when LIMIT_BYTES_TO_FORMAT is nonzero.  */
-static off_t max_bytes_to_format;
+/* The maximum number of bytes that will be formatted.  */
+static uintmax_t max_bytes_to_format;
+
+/* The offset of the first byte after the last byte to be formatted.  */
+static uintmax_t end_offset;
 
 /* When nonzero and two or more consecutive blocks are equal, format
    only the first block and output an asterisk alone on the following
@@ -214,11 +217,13 @@ static size_t bytes_per_block;
    It differs from *file_list only when *file_list is "-".  */
 static char const *input_filename;
 
-/* A NULL-terminated list of the file-arguments from the command line.
-   If no file-arguments were specified, this variable is initialized
-   to { "-", NULL }.  */
+/* A NULL-terminated list of the file-arguments from the command line.  */
 static char const *const *file_list;
 
+/* Initializer for file_list if no file-arguments
+   were specified on the command line.  */
+static char const *const default_file_list[] = {"-", NULL};
+
 /* The input stream associated with the current file.  */
 static FILE *in_stream;
 
@@ -271,13 +276,15 @@ Usage: %s [OPTION]... [FILE]...\n\
   or:  %s --traditional [FILE] [[+]OFFSET [[+]LABEL]]\n\
 "),
 	      program_name, program_name);
-      printf (_("\
-Write an unambiguous representation, octal bytes by default, of FILE\n\
-to standard output.  With no FILE, or when FILE is -, read standard input.\n\
+      printf (_("\n\
+Write an unambiguous representation, octal bytes by default,\n\
+of FILE to standard output.  With more than one FILE argument,\n\
+concatenate them in the listed order to form the input.\n\
+With no FILE, or when FILE is -, read standard input.\n\
 \n\
   -A, --address-radix=RADIX   decide how file offsets are printed\n\
-  -j, --skip-bytes=BYTES      skip BYTES input bytes first on each file\n\
-  -N, --read-bytes=BYTES      limit dump to BYTES input bytes per file\n\
+  -j, --skip-bytes=BYTES      skip BYTES input bytes first\n\
+  -N, --read-bytes=BYTES      limit dump to BYTES input bytes\n\
   -s, --strings[=BYTES]       output strings of at least BYTES graphic chars\n\
   -t, --format=TYPE           select output format or formats\n\
   -v, --output-duplicates     do not use * to mark line suppression\n\
@@ -360,9 +367,9 @@ lcm (unsigned int u, unsigned int v)
 }
 
 static void
-print_s_char (off_t n_bytes, const char *block, const char *fmt_string)
+print_s_char (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes; i > 0; i--)
     {
       int tmp = (unsigned) *(const unsigned char *) block;
@@ -375,9 +382,9 @@ print_s_char (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_char (off_t n_bytes, const char *block, const char *fmt_string)
+print_char (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes; i > 0; i--)
     {
       unsigned int tmp = *(const unsigned char *) block;
@@ -387,9 +394,9 @@ print_char (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_s_short (off_t n_bytes, const char *block, const char *fmt_string)
+print_s_short (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (unsigned short); i > 0; i--)
     {
       int tmp = (unsigned) *(const unsigned short *) block;
@@ -402,9 +409,9 @@ print_s_short (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_short (off_t n_bytes, const char *block, const char *fmt_string)
+print_short (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (unsigned short); i > 0; i--)
     {
       unsigned int tmp = *(const unsigned short *) block;
@@ -414,9 +421,9 @@ print_short (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_int (off_t n_bytes, const char *block, const char *fmt_string)
+print_int (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (unsigned int); i > 0; i--)
     {
       unsigned int tmp = *(const unsigned int *) block;
@@ -426,9 +433,9 @@ print_int (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_long (off_t n_bytes, const char *block, const char *fmt_string)
+print_long (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (unsigned long); i > 0; i--)
     {
       unsigned long tmp = *(const unsigned long *) block;
@@ -438,9 +445,9 @@ print_long (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_long_long (off_t n_bytes, const char *block, const char *fmt_string)
+print_long_long (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (ulonglong_t); i > 0; i--)
     {
       ulonglong_t tmp = *(const ulonglong_t *) block;
@@ -450,9 +457,9 @@ print_long_long (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_float (off_t n_bytes, const char *block, const char *fmt_string)
+print_float (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (float); i > 0; i--)
     {
       float tmp = *(const float *) block;
@@ -462,9 +469,9 @@ print_float (off_t n_bytes, const char *block, const char *fmt_string)
 }
 
 static void
-print_double (off_t n_bytes, const char *block, const char *fmt_string)
+print_double (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (double); i > 0; i--)
     {
       double tmp = *(const double *) block;
@@ -475,9 +482,9 @@ print_double (off_t n_bytes, const char *block, const char *fmt_string)
 
 #ifdef HAVE_LONG_DOUBLE
 static void
-print_long_double (off_t n_bytes, const char *block, const char *fmt_string)
+print_long_double (size_t n_bytes, const char *block, const char *fmt_string)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes / sizeof (LONG_DOUBLE); i > 0; i--)
     {
       LONG_DOUBLE tmp = *(const LONG_DOUBLE *) block;
@@ -489,9 +496,9 @@ print_long_double (off_t n_bytes, const char *block, const char *fmt_string)
 #endif
 
 static void
-dump_hexl_mode_trailer (off_t n_bytes, const char *block)
+dump_hexl_mode_trailer (size_t n_bytes, const char *block)
 {
-  off_t i;
+  size_t i;
   fputs ("  >", stdout);
   for (i = n_bytes; i > 0; i--)
     {
@@ -504,10 +511,10 @@ dump_hexl_mode_trailer (off_t n_bytes, const char *block)
 }
 
 static void
-print_named_ascii (off_t n_bytes, const char *block,
+print_named_ascii (size_t n_bytes, const char *block,
 		   const char *unused_fmt_string ATTRIBUTE_UNUSED)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes; i > 0; i--)
     {
       unsigned int c = *(const unsigned char *) block;
@@ -531,10 +538,10 @@ print_named_ascii (off_t n_bytes, const char *block,
 }
 
 static void
-print_ascii (off_t n_bytes, const char *block,
+print_ascii (size_t n_bytes, const char *block,
 	     const char *unused_fmt_string ATTRIBUTE_UNUSED)
 {
-  off_t i;
+  size_t i;
   for (i = n_bytes; i > 0; i--)
     {
       unsigned int c = *(const unsigned char *) block;
@@ -636,7 +643,7 @@ decode_one_format (const char *s_orig, const char *s, const char **next,
   enum output_format fmt;
   const char *pre_fmt_string;
   char *fmt_string;
-  void (*print_function) PARAMS ((off_t, const char *, const char *));
+  void (*print_function) PARAMS ((size_t, const char *, const char *));
   const char *p;
   unsigned int c;
   unsigned int field_width = 0;
@@ -892,6 +899,88 @@ this system doesn't provide a %lu-byte floating point type"), s_orig, size);
   return 0;
 }
 
+/* Given a list of one or more input filenames FILE_LIST, set the global
+   file pointer IN_STREAM and the global string INPUT_FILENAME to the
+   first one that can be successfully opened. Modify FILE_LIST to
+   reference the next filename in the list.  A file name of "-" is
+   interpreted as standard input.  If any file open fails, give an error
+   message and return nonzero.  */
+
+static int
+open_next_file (void)
+{
+  int err = 0;
+
+  do
+    {
+      input_filename = *file_list;
+      if (input_filename == NULL)
+	return err;
+      ++file_list;
+
+      if (STREQ (input_filename, "-"))
+	{
+	  input_filename = _("standard input");
+	  in_stream = stdin;
+	  have_read_stdin = 1;
+	}
+      else
+	{
+	  in_stream = fopen (input_filename, "r");
+	  if (in_stream == NULL)
+	    {
+	      error (0, errno, "%s", input_filename);
+	      err = 1;
+	    }
+	}
+    }
+  while (in_stream == NULL);
+
+  if (limit_bytes_to_format && !flag_dump_strings)
+    SETVBUF (in_stream, NULL, _IONBF, 0);
+  SET_BINARY (fileno (in_stream));
+
+  return err;
+}
+
+/* Test whether there have been errors on in_stream, and close it if
+   it is not standard input.  Return nonzero if there has been an error
+   on in_stream or stdout; return zero otherwise.  This function will
+   report more than one error only if both a read and a write error
+   have occurred.  */
+
+static int
+check_and_close (void)
+{
+  int err = 0;
+
+  if (in_stream != NULL)
+    {
+      if (ferror (in_stream))
+	{
+	  error (0, errno, "%s", input_filename);
+	  if (in_stream != stdin)
+	    fclose (in_stream);
+	  err = 1;
+	}
+      else if (in_stream != stdin && fclose (in_stream) == EOF)
+	{
+	  error (0, errno, "%s", input_filename);
+	  err = 1;
+	}
+
+      in_stream = NULL;
+    }
+
+  if (ferror (stdout))
+    {
+      error (0, errno, _("standard output"));
+      err = 1;
+    }
+
+  return err;
+}
+
 /* Decode the POSIX-style od format string S.  Append the decoded
    representation to the global array SPEC, reallocating SPEC if
    necessary.  Return zero if S is valid, nonzero otherwise.  */
@@ -933,42 +1022,20 @@ decode_format_string (const char *s)
    file pointer IN_STREAM to position N_SKIP in the concatenation of
    those files.  If any file operation fails or if there are fewer than
    N_SKIP bytes in the combined input, give an error message and return
-   nonzero.  When possible, use seek- rather than read operations to
-   advance IN_STREAM.  A file name of "-" is interpreted as standard
-   input.  */
+   nonzero.  When possible, use seek rather than read operations to
+   advance IN_STREAM.  */
 
 static int
-skip (off_t n_skip)
+skip (uintmax_t n_skip)
 {
-  int err;
+  int err = 0;
 
-  err = 0;
-  for ( /* empty */ ; *file_list != NULL; ++file_list)
+  if (n_skip == 0)
+    return 0;
+
+  while (in_stream != NULL)	/* EOF.  */
     {
       struct stat file_stats;
-      int j;
-
-      if (STREQ (*file_list, "-"))
-	{
-	  input_filename = _("standard input");
-	  in_stream = stdin;
-	  have_read_stdin = 1;
-	}
-      else
-	{
-	  input_filename = *file_list;
-	  in_stream = fopen (input_filename, "r");
-	  if (in_stream == NULL)
-	    {
-	      error (0, errno, "%s", input_filename);
-	      err = 1;
-	      continue;
-	    }
-	}
-      SET_BINARY (fileno (in_stream));
-
-      if (n_skip == 0)
-	break;
 
       /* First try seeking.  For large offsets, this extra work is
 	 worthwhile.  If the offset is below some threshold it may be
@@ -980,58 +1047,61 @@ skip (off_t n_skip)
 	     Try to do that by getting file's size using fstat.
 	     But that will work only for regular files.  */
 
-      if (fstat (fileno (in_stream), &file_stats))
+      if (fstat (fileno (in_stream), &file_stats) == 0)
 	{
-	  error (0, errno, "%s", input_filename);
-	  err = 1;
-	  continue;
-	}
+	  /* The st_size field is valid only for regular files
+	     (and for symbolic links, which cannot occur here).
+	     If the number of bytes left to skip is at least
+	     as large as the size of the current file, we can
+	     decrement n_skip and go on to the next file.  */
 
-      /* The st_size field is valid only for regular files
-	 (and for symbolic links, which cannot occur here).
-	 If the number of bytes left to skip is at least as large as
-	 the size of the current file, we can decrement
-	 n_skip and go on to the next file.  */
-      if (S_ISREG (file_stats.st_mode))
-	{
-	  if (file_stats.st_size <= n_skip)
+	  if (S_ISREG (file_stats.st_mode) && 0 <= file_stats.st_size)
 	    {
-	      n_skip -= file_stats.st_size;
-	      if (in_stream != stdin && fclose (in_stream) == EOF)
+	      if (file_stats.st_size <= n_skip)
+		n_skip -= file_stats.st_size;
+	      else
 		{
-		  error (0, errno, "%s", input_filename);
-		  err = 1;
+		  if (lseek (fileno (in_stream), n_skip, SEEK_CUR) < 0)
+		    {
+		      error (0, errno, "%s", input_filename);
+		      err = 1;
+		    }
+		  n_skip = 0;
 		}
-	      continue;
 	    }
+
+	  /* If it's not a regular file with nonnegative size,
+	     position the file pointer by reading.  */
+
 	  else
 	    {
-	      if (0 <= lseek (fileno (in_stream), n_skip, SEEK_CUR))
+	      char buf[BUFSIZ];
+	      size_t n_bytes_read, n_bytes_to_read = BUFSIZ;
+
+	      while (0 < n_skip)
 		{
-		  n_skip = 0;
-		  break;
+		  if (n_skip < n_bytes_to_read)
+		    n_bytes_to_read = n_skip;
+		  n_bytes_read = fread (buf, 1, n_bytes_to_read, in_stream);
+		  n_skip -= n_bytes_read;
+		  if (n_bytes_read != n_bytes_to_read)
+		    break;
 		}
 	    }
-	}
 
-      /* Seek didn't work or wasn't attempted;  position the file pointer
-	 by reading.  */
-
-      for (j = n_skip / BUFSIZ; 0 <= j; j--)
-	{
-	  char buf[BUFSIZ];
-	  size_t n_bytes_to_read = (0 < j
-				    ? BUFSIZ
-				    : n_skip % BUFSIZ);
-	  size_t n_bytes_read;
-	  n_bytes_read = fread (buf, 1, n_bytes_to_read, in_stream);
-	  n_skip -= n_bytes_read;
-	  if (n_bytes_read != n_bytes_to_read)
+	  if (n_skip == 0)
 	    break;
 	}
 
-      if (n_skip == 0)
-	break;
+      else   /* cannot fstat() file */
+	{
+	  error (0, errno, "%s", input_filename);
+	  err = 1;
+	}
+
+      err |= check_and_close ();
+
+      err |= open_next_file ();
     }
 
   if (n_skip != 0)
@@ -1041,12 +1111,12 @@ skip (off_t n_skip)
 }
 
 static void
-format_address_none (off_t address ATTRIBUTE_UNUSED, char c ATTRIBUTE_UNUSED)
+format_address_none (uintmax_t address ATTRIBUTE_UNUSED, char c ATTRIBUTE_UNUSED)
 {
 }
 
 static void
-format_address_std (off_t address, char c)
+format_address_std (uintmax_t address, char c)
 {
   char buf[MAX_ADDRESS_LENGTH + 2];
   char *p = buf + sizeof buf;
@@ -1086,7 +1156,7 @@ format_address_std (off_t address, char c)
 }
 
 static void
-format_address_paren (off_t address, char c)
+format_address_paren (uintmax_t address, char c)
 {
   putchar ('(');
   format_address_std (address, ')');
@@ -1094,7 +1164,7 @@ format_address_paren (off_t address, char c)
 }
 
 static void
-format_address_label (off_t address, char c)
+format_address_label (uintmax_t address, char c)
 {
   format_address_std (address, ' ');
   format_address_paren (address + pseudo_offset, c);
@@ -1112,7 +1182,7 @@ format_address_label (off_t address, char c)
    only when it has not been padded to length BYTES_PER_BLOCK.  */
 
 static void
-write_block (off_t current_offset, off_t n_bytes,
+write_block (uintmax_t current_offset, size_t n_bytes,
 	     const char *prev_block, const char *curr_block)
 {
   static int first = 1;
@@ -1162,119 +1232,57 @@ write_block (off_t current_offset, off_t n_bytes,
   first = 0;
 }
 
-/* Test whether there have been errors on in_stream, and close it if
-   it is not standard input.  Return nonzero if there has been an error
-   on in_stream or stdout; return zero otherwise.  This function will
-   report more than one error only if both a read and a write error
-   have occurred.  */
+/* Read a single byte into *C from the concatenation of the input files
+   named in the global array FILE_LIST.  On the first call to this
+   function, the global variable IN_STREAM is expected to be an open
+   stream associated with the input file INPUT_FILENAME.  If IN_STREAM
+   is at end-of-file, close it and update the global variables IN_STREAM
+   and INPUT_FILENAME so they correspond to the next file in the list.
+   Then try to read a byte from the newly opened file.  Repeat if
+   necessary until EOF is reached for the last file in FILE_LIST, then
+   set *C to EOF and return.  Subsequent calls do likewise.  The return
+   value is nonzero if any errors occured, zero otherwise.  */
 
 static int
-check_and_close (void)
+read_char (int *c)
 {
-  int err;
+  int err = 0;
 
-  err = 0;
-  if (ferror (in_stream))
-    {
-      error (0, errno, "%s", input_filename);
-      if (in_stream != stdin)
-	fclose (in_stream);
-      err = 1;
-    }
-  else if (in_stream != stdin && fclose (in_stream) == EOF)
-    {
-      error (0, errno, "%s", input_filename);
-      err = 1;
-    }
+  *c = EOF;
 
-  if (ferror (stdout))
+  while (in_stream != NULL)	/* EOF.  */
     {
-      error (0, errno, _("standard output"));
-      err = 1;
+      *c = fgetc (in_stream);
+
+      if (*c != EOF)
+	break;
+
+      err |= check_and_close ();
+
+      err |= open_next_file ();
     }
 
   return err;
 }
 
-/* Read a single byte into *C from the concatenation of the input files
-   named in the global array FILE_LIST.  On the first call to this
-   function, the global variable IN_STREAM is expected to be an open
-   stream associated with the input file *FILE_LIST.  If IN_STREAM is
-   at end-of-file, close it and update the global variables IN_STREAM,
-   FILE_LIST, and INPUT_FILENAME so they correspond to the next file in
-   the list.  Then try to read a byte from the newly opened file.
-   Repeat if necessary until *FILE_LIST is NULL.  When EOF is reached
-   for the last file in FILE_LIST, set *C to EOF and return.  Subsequent
-   calls do likewise.  The return value is nonzero if any errors
-   occured, zero otherwise.  */
-
-static int
-read_char (int *c)
-{
-  int err;
-
-  if (*file_list == NULL)
-    {
-      *c = EOF;
-      return 0;
-    }
-
-  err = 0;
-  while (1)
-    {
-      *c = fgetc (in_stream);
-
-      if (*c != EOF)
-	return err;
-
-      err |= check_and_close ();
-
-      do
-	{
-	  ++file_list;
-	  if (*file_list == NULL)
-	    return err;
-
-	  if (STREQ (*file_list, "-"))
-	    {
-	      input_filename = _("standard input");
-	      in_stream = stdin;
-	      have_read_stdin = 1;
-	    }
-	  else
-	    {
-	      input_filename = *file_list;
-	      in_stream = fopen (input_filename, "r");
-	      if (in_stream == NULL)
-		{
-		  error (0, errno, "%s", input_filename);
-		  err = 1;
-		}
-	    }
-	  SET_BINARY (fileno (in_stream));
-	}
-      while (in_stream == NULL);
-    }
-}
-
 /* Read N bytes into BLOCK from the concatenation of the input files
    named in the global array FILE_LIST.  On the first call to this
    function, the global variable IN_STREAM is expected to be an open
-   stream associated with the input file *FILE_LIST.  On subsequent
-   calls, if *FILE_LIST is NULL, don't modify BLOCK and return zero.
-   If all N bytes cannot be read from IN_STREAM, close IN_STREAM and
-   update the global variables IN_STREAM, FILE_LIST, and INPUT_FILENAME.
-   Then try to read the remaining bytes from the newly opened file.
-   Repeat if necessary until *FILE_LIST is NULL.  Set *N_BYTES_IN_BUFFER
-   to the number of bytes read.  If an error occurs, it will be detected
-   through ferror when the stream is about to be closed.  If there is an
-   error, give a message but continue reading as usual and return nonzero.
-   Otherwise return zero.  */
+   stream associated with the input file INPUT_FILENAME.  If all N
+   bytes cannot be read from IN_STREAM, close IN_STREAM and update
+   the global variables IN_STREAM and INPUT_FILENAME.  Then try to
+   read the remaining bytes from the newly opened file.  Repeat if
+   necessary until EOF is reached for the last file in FILE_LIST.
+   On subsequent calls, don't modify BLOCK and return zero.  Set
+   *N_BYTES_IN_BUFFER to the number of bytes read.  If an error occurs,
+   it will be detected through ferror when the stream is about to be
+   closed.  If there is an error, give a message but continue reading
+   as usual and return nonzero.  Otherwise return zero.  */
 
 static int
 read_block (size_t n, char *block, size_t *n_bytes_in_buffer)
 {
-  int err;
+  int err = 0;
 
   assert (0 < n && n <= bytes_per_block);
 
@@ -1283,11 +1291,7 @@ read_block (size_t n, char *block, size_t *n_bytes_in_buffer)
   if (n == 0)
     return 0;
 
-  if (*file_list == NULL)
-    return 0;			/* EOF.  */
-
-  err = 0;
-  while (1)
+  while (in_stream != NULL)	/* EOF.  */
     {
       size_t n_needed;
       size_t n_read;
@@ -1298,36 +1302,14 @@ read_block (size_t n, char *block, size_t *n_bytes_in_buffer)
       *n_bytes_in_buffer += n_read;
 
       if (n_read == n_needed)
-	return err;
+	break;
 
       err |= check_and_close ();
 
-      do
-	{
-	  ++file_list;
-	  if (*file_list == NULL)
-	    return err;
-
-	  if (STREQ (*file_list, "-"))
-	    {
-	      input_filename = _("standard input");
-	      in_stream = stdin;
-	      have_read_stdin = 1;
-	    }
-	  else
-	    {
-	      input_filename = *file_list;
-	      in_stream = fopen (input_filename, "r");
-	      if (in_stream == NULL)
-		{
-		  error (0, errno, "%s", input_filename);
-		  err = 1;
-		}
-	    }
-	  SET_BINARY (fileno (in_stream));
-	}
-      while (in_stream == NULL);
+      err |= open_next_file ();
     }
+
+  return err;
 }
 
 /* Return the least common multiple of the sizes associated
@@ -1345,18 +1327,16 @@ get_lcm (void)
 }
 
 /* If S is a valid pre-POSIX offset specification with an optional leading '+'
-   return the offset it denotes.  Otherwise, return -1.  */
+   return nonzero and set *OFFSET to the offset it denotes.  */
 
-off_t
-parse_old_offset (const char *s)
+int
+parse_old_offset (const char *s, uintmax_t *offset)
 {
   int radix;
-  off_t offset;
   enum strtol_error s_err;
-  uintmax_t tmp;
 
   if (*s == '\0')
-    return -1;
+    return 0;
 
   /* Skip over any leading '+'. */
   if (s[0] == '+')
@@ -1375,17 +1355,13 @@ parse_old_offset (const char *s)
 	radix = 8;
     }
 
-  s_err = xstrtoumax (s, NULL, radix, &tmp, "Bb");
+  s_err = xstrtoumax (s, NULL, radix, offset, "Bb");
   if (s_err != LONGINT_OK)
     {
       STRTOL_FAIL_WARN (s, _("old-style offset"), s_err);
-      return -1;
+      return 0;
     }
-  if (OFF_T_MAX < tmp)
-    error (EXIT_FAILURE, 0,
-	   _("%s is larger than the maximum file size on this system"), s);
-  offset = tmp;
-  return offset;
+  return 1;
 }
 
 /* Read a chunk of size BYTES_PER_BLOCK from the input files, write the
@@ -1404,8 +1380,7 @@ static int
 dump (void)
 {
   char *block[2];
-  off_t current_offset;
-  off_t end_offset IF_LINT (= 0);
+  uintmax_t current_offset;
   int idx;
   int err;
   size_t n_bytes_read;
@@ -1419,8 +1394,6 @@ dump (void)
   err = 0;
   if (limit_bytes_to_format)
     {
-      end_offset = n_bytes_to_skip + max_bytes_to_format;
-
       while (1)
 	{
 	  size_t n_needed;
@@ -1430,7 +1403,7 @@ dump (void)
 	      break;
 	    }
 	  n_needed = MIN (end_offset - current_offset,
-			  (off_t) bytes_per_block);
+			  (uintmax_t) bytes_per_block);
 	  err |= read_block (n_needed, block[idx], &n_bytes_read);
 	  if (n_bytes_read < bytes_per_block)
 	    break;
@@ -1475,7 +1448,7 @@ dump (void)
 
   format_address (current_offset, '\n');
 
-  if (limit_bytes_to_format && current_offset > end_offset)
+  if (limit_bytes_to_format && current_offset >= end_offset)
     err |= check_and_close ();
 
   return err;
@@ -1493,7 +1466,7 @@ dump_strings (void)
 {
   size_t bufsize = MAX (100, string_min);
   char *buf = xmalloc (bufsize);
-  off_t address = n_bytes_to_skip;
+  uintmax_t address = n_bytes_to_skip;
   int err;
 
   err = 0;
@@ -1506,7 +1479,7 @@ dump_strings (void)
     tryline:
 
       if (limit_bytes_to_format
-	  && address >= (n_bytes_to_skip + max_bytes_to_format - string_min))
+	  && (end_offset < string_min || end_offset - string_min <= address))
 	break;
 
       for (i = 0; i < string_min; i++)
@@ -1526,8 +1499,7 @@ dump_strings (void)
 
       /* We found a run of `string_min' printable characters.
 	 Now see if it is terminated with a null byte.  */
-      while (!limit_bytes_to_format
-	     || address < n_bytes_to_skip + max_bytes_to_format)
+      while (!limit_bytes_to_format || address < end_offset)
 	{
 	  if (i == bufsize)
 	    {
@@ -1593,7 +1565,7 @@ dump_strings (void)
     }
 
   /* We reach this point only if we search through
-     (max_bytes_to_format - string_min) bytes before reachine EOF.  */
+     (max_bytes_to_format - string_min) bytes before reaching EOF.  */
 
   free (buf);
 
@@ -1615,7 +1587,7 @@ main (int argc, char **argv)
 
   /* The old-style `pseudo starting address' to be printed in parentheses
      after any true address.  */
-  off_t pseudo_start IF_LINT (= 0);
+  uintmax_t pseudo_start IF_LINT (= 0);
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
@@ -1699,8 +1671,7 @@ it must be one character from [doxn]"),
 	  break;
 
 	case 'j':
-	  s_err = xstrtoumax (optarg, NULL, 0, &tmp, "bkm");
-	  n_bytes_to_skip = tmp;
+	  s_err = xstrtoumax (optarg, NULL, 0, &n_bytes_to_skip, "bkm");
 	  if (s_err != LONGINT_OK)
 	    STRTOL_FATAL_ERROR (optarg, _("skip argument"), s_err);
 	  break;
@@ -1708,15 +1679,9 @@ it must be one character from [doxn]"),
 	case 'N':
 	  limit_bytes_to_format = 1;
 
-	  s_err = xstrtoumax (optarg, NULL, 0, &tmp, "bkm");
-	  max_bytes_to_format = tmp;
+	  s_err = xstrtoumax (optarg, NULL, 0, &max_bytes_to_format, "bkm");
 	  if (s_err != LONGINT_OK)
 	    STRTOL_FATAL_ERROR (optarg, _("limit argument"), s_err);
-
-	  if (OFF_T_MAX < tmp)
-	    error (EXIT_FAILURE, 0,
-		   _("%s is larger than the maximum file size on this system"),
-		   optarg);
 	  break;
 
 	case 's':
@@ -1822,11 +1787,11 @@ it must be one character from [doxn]"),
 
   if (traditional)
     {
-      off_t offset;
+      uintmax_t offset;
 
       if (n_files == 1)
 	{
-	  if ((offset = parse_old_offset (argv[optind])) >= 0)
+	  if (parse_old_offset (argv[optind], &offset))
 	    {
 	      n_bytes_to_skip = offset;
 	      --n_files;
@@ -1835,9 +1800,9 @@ it must be one character from [doxn]"),
 	}
       else if (n_files == 2)
 	{
-	  off_t o1, o2;
-	  if ((o1 = parse_old_offset (argv[optind])) >= 0
-	      && (o2 = parse_old_offset (argv[optind + 1])) >= 0)
+	  uintmax_t o1, o2;
+	  if (parse_old_offset (argv[optind], &o1)
+	      && parse_old_offset (argv[optind + 1], &o2))
 	    {
 	      n_bytes_to_skip = o1;
 	      flag_pseudo_start = 1;
@@ -1845,7 +1810,7 @@ it must be one character from [doxn]"),
 	      argv += 2;
 	      n_files -= 2;
 	    }
-	  else if ((o2 = parse_old_offset (argv[optind + 1])) >= 0)
+	  else if (parse_old_offset (argv[optind + 1], &o2))
 	    {
 	      n_bytes_to_skip = o2;
 	      --n_files;
@@ -1862,9 +1827,9 @@ it must be one character from [doxn]"),
 	}
       else if (n_files == 3)
 	{
-	  off_t o1, o2;
-	  if ((o1 = parse_old_offset (argv[optind + 1])) >= 0
-	      && (o2 = parse_old_offset (argv[optind + 2])) >= 0)
+	  uintmax_t o1, o2;
+	  if (parse_old_offset (argv[optind + 1], &o1)
+	      && parse_old_offset (argv[optind + 2], &o2))
 	    {
 	      n_bytes_to_skip = o1;
 	      flag_pseudo_start = 1;
@@ -1900,6 +1865,13 @@ it must be one character from [doxn]"),
 	}
     }
 
+  if (limit_bytes_to_format)
+    {
+      end_offset = n_bytes_to_skip + max_bytes_to_format;
+      if (end_offset < n_bytes_to_skip)
+	error (EXIT_FAILURE, 0, "skip-bytes + read-bytes is too large");
+    }
+
   if (n_specs == 0)
     {
       if (decode_one_format ("o2", "o2", NULL, &(spec[0])))
@@ -1913,17 +1885,27 @@ it must be one character from [doxn]"),
     }
 
   if (n_files > 0)
-    file_list = (char const *const *) &argv[optind];
+    {
+      /* Set the global pointer FILE_LIST so that it
+	 references the first file-argument on the command-line.  */
+
+      file_list = (char const *const *) &argv[optind];
+    }
   else
     {
-      /* If no files were listed on the command line, set up the
-	 global array FILE_LIST so that it contains the null-terminated
-	 list of one name: "-".  */
-      static char const *const default_file_list[] = {"-", NULL};
+      /* No files were listed on the command line.
+	 Set the global pointer FILE_LIST so that it
+	 references the null-terminated list of one name: "-".  */
 
       file_list = default_file_list;
     }
 
+  /* open the first input file */
+  err |= open_next_file ();
+  if (in_stream == NULL)
+    goto cleanup;
+
+  /* skip over any unwanted header bytes */
   err |= skip (n_bytes_to_skip);
   if (in_stream == NULL)
     goto cleanup;
