@@ -243,22 +243,24 @@ With more than one FILE, precede each with a header giving the file name.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
       --retry              keep trying to open a file even if it is\n\
-                             inaccessible when tail starts or if it becomes\n\
-                             inaccessible later -- useful only with -f\n\
+                           inaccessible when tail starts or if it becomes\n\
+                           inaccessible later -- useful only with -f\n\
   -c, --bytes=N            output the last N bytes\n\
-  -f, --follow[={name|descriptor}] output appended data as the file grows;\n\
-                             -f, --follow, and --follow=descriptor are\n\
-                             equivalent\n\
+  -f, --follow[={name|descriptor}]\n\
+                           output appended data as the file grows;\n\
+                           -f, --follow, and --follow=descriptor are\n\
+                           equivalent\n\
+  -F                       same as --follow=name --retry\n\
   -n, --lines=N            output the last N lines, instead of the last %d\n\
       --max-unchanged-stats=N\n\
                            with --follow=name, reopen a FILE which has not\n\
-                             changed size after N (default %d) iterations\n\
-                             to see if it has been unlinked or renamed\n\
-                             (this is the usual case of rotated log files)\n\
+                           changed size after N (default %d) iterations\n\
+                           to see if it has been unlinked or renamed\n\
+                           (this is the usual case of rotated log files)\n\
       --pid=PID            with -f, terminate after process ID, PID dies\n\
   -q, --quiet, --silent    never output headers giving file names\n\
   -s, --sleep-interval=S   with -f, each iteration lasts approximately S\n\
-                             (default 1) seconds\n\
+                           (default 1) seconds\n\
   -v, --verbose            always output headers giving file names\n\
       --help               display this help and exit\n\
       --version            output version information and exit\n\
@@ -1253,7 +1255,7 @@ parse_obsolescent_option (int argc, const char *const *argv,
   n_string_end = p;
 
   t_count_lines = 1;
-  if (*p == 'c')
+  if (*p == 'c' || *p == 'b')
     {
       t_count_lines = 0;
       ++p;
@@ -1296,7 +1298,9 @@ parse_obsolescent_option (int argc, const char *const *argv,
       strtol_error s_err;
       unsigned long int tmp_ulong;
       char *end;
-      s_err = xstrtoul (n_string, &end, 10, &tmp_ulong, NULL);
+
+      s_err = xstrtoul (n_string, &end, 10, &tmp_ulong,
+			*n_string_end == 'b' ? "b" : NULL);
       if (s_err == LONGINT_OK && tmp_ulong <= OFF_T_MAX)
 	*n_units = (off_t) tmp_ulong;
       else
@@ -1310,7 +1314,7 @@ parse_obsolescent_option (int argc, const char *const *argv,
 
 	  error (0, 0,
 		 _("%s: %s is so large that it is not representable"),
-		 n_string_tmp, (count_lines
+		 n_string_tmp, (t_count_lines
 				? _("number of lines")
 				: _("number of bytes")));
 	  free (n_string_tmp);
@@ -1362,7 +1366,7 @@ parse_options (int argc, char **argv,
   count_lines = 1;
   forever = from_start = print_headers = 0;
 
-  while ((c = getopt_long (argc, argv, "c:n:fqs:v", long_options, NULL))
+  while ((c = getopt_long (argc, argv, "c:n:fFqs:v", long_options, NULL))
 	 != -1)
     {
       switch (c)
@@ -1370,6 +1374,11 @@ parse_options (int argc, char **argv,
 	case 0:
 	  break;
 
+	case 'F':
+	  forever = 1;
+	  follow_mode = Follow_name;
+	  reopen_inaccessible_files = 1;
+
 	case 'c':
 	case 'n':
 	  count_lines = (c == 'n');
