@@ -25,6 +25,7 @@
 
 #include "system.h"
 #include "closeout.h"
+#include "argmatch.h"
 #include "linebuffer.h"
 #include "error.h"
 #include "xstrtol.h"
@@ -33,7 +34,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "uniq"
 
-#define AUTHORS "Richard Stallman and David MacKenzie"
+#define AUTHORS N_ ("Richard Stallman and David MacKenzie")
 
 #define SWAP_LINES(A, B)			\
   do						\
@@ -49,13 +50,13 @@
 char *program_name;
 
 /* Number of fields to skip on each line when doing comparisons. */
-static int skip_fields;
+static size_t skip_fields;
 
 /* Number of chars to skip after skipping any fields. */
-static int skip_chars;
+static size_t skip_chars;
 
-/* Number of chars to compare; if 0, compare the whole lines. */
-static int check_chars;
+/* Number of chars to compare. */
+static size_t check_chars;
 
 enum countmode
 {
@@ -81,11 +82,36 @@ static enum output_mode mode;
 /* If nonzero, ignore case when comparing.  */
 static int ignore_case;
 
+enum delimit_method
+{
+  /* No delimiters output.  --all-repeated[=none] */
+  DM_NONE,
+
+  /* Delimiter precedes all groups.  --all-repeated=precede */
+  DM_PRECEDE,
+
+  /* Delimit all groups.  --all-repeated=separate */
+  DM_SEPARATE
+};
+
+static char const *const delimit_method_string[] =
+{
+  "none", "precede", "separate", 0
+};
+
+static enum delimit_method const delimit_method_map[] =
+{
+  DM_NONE, DM_PRECEDE, DM_SEPARATE
+};
+
+/* Select whether/how to delimit groups of duplicate lines.  */
+static enum delimit_method delimit_groups;
+
 static struct option const longopts[] =
 {
   {"count", no_argument, NULL, 'c'},
   {"repeated", no_argument, NULL, 'd'},
-  {"all-repeated", no_argument, NULL, 'D'},
+  {"all-repeated", optional_argument, NULL, 'D'},
   {"ignore-case", no_argument, NULL, 'i'},
   {"unique", no_argument, NULL, 'u'},
   {"skip-fields", required_argument, NULL, 'f'},
@@ -114,7 +140,9 @@ standard input), writing to OUTPUT (or standard output).\n\
 \n\
   -c, --count           prefix lines by the number of occurrences\n\
   -d, --repeated        only print duplicate lines\n\
-  -D, --all-repeated    print all duplicate lines\n\
+  -D, --all-repeated[=delimit-method] print all duplicate lines\n\
+                        delimit-method={all,minimum,none(default)}\n\
+                        Delimiting is done with blank lines.\n\
   -f, --skip-fields=N   avoid comparing the first N fields\n\
   -i, --ignore-case     ignore differences in case when comparing\n\
   -s, --skip-chars=N    avoid comparing the first N characters\n\
@@ -133,15 +161,28 @@ Fields are skipped before chars.\n\
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
+/* Convert OPT to size_t, reporting an error using MSGID if it does
+   not fit.  */
+
+static size_t
+size_opt (char const *opt, char const *msgid)
+{
+  unsigned long int size;
+  if (xstrtoul (opt, NULL, 10, &size, "") != LONGINT_OK
+      || SIZE_MAX < size)
+    error (EXIT_FAILURE, 0, "%s: %s", opt, _(msgid));
+  return size;
+}
+
 /* Given a linebuffer LINE,
    return a pointer to the beginning of the line's field to be compared. */
 
 static char *
 find_field (const struct linebuffer *line)
 {
-  register int count;
+  register size_t count;
   register char *lp = line->buffer;
-  register size_t size = line->length;
+  register size_t size = line->length - 1;
   register size_t i = 0;
 
   for (count = 0; count < skip_fields && i < size; count++)
@@ -166,27 +207,21 @@ find_field (const struct linebuffer *line)
 static int
 different (const char *old, const char *new, size_t oldlen, size_t newlen)
 {
-  register int order;
+  if (check_chars < oldlen)
+    oldlen = check_chars;
+  if (check_chars < newlen)
+    newlen = check_chars;
 
-  if (check_chars)
-    {
-      if (oldlen > check_chars)
-	oldlen = check_chars;
-      if (newlen > check_chars)
-	newlen = check_chars;
-    }
+  if (oldlen != newlen)
+    return 1;
 
   /* Use an if-statement here rather than a function variable to
      avoid portability hassles of getting a non-conflicting declaration
      of memcmp.  */
   if (ignore_case)
-    order = memcasecmp (old, new, MIN (oldlen, newlen));
+    return memcasecmp (old, new, oldlen);
   else
-    order = memcmp (old, new, MIN (oldlen, newlen));
-
-  if (order == 0)
-    return oldlen - newlen;
-  return order;
+    return memcmp (old, new, oldlen);
 }
 
 /* Output the line in linebuffer LINE to stream STREAM
@@ -276,6 +311,7 @@ check_file (const char *infile, const char *outfile)
       char *prevfield;
       size_t prevlen;
       int match_count = 0;
+      int first_delimiter = 1;
 
       if (readline (prevline, istream) == 0)
 	goto closefiles;
@@ -296,6 +332,22 @@ check_file (const char *infile, const char *outfile)
 	  if (match)
 	    ++match_count;
 
+          if (mode == output_all_repeated && delimit_groups != DM_NONE)
+	    {
+	      if (!match)
+		{
+		  if (match_count) /* a previous match */
+		    first_delimiter = 0; /* Only used when DM_SEPARATE */
+		}
+	      else if (match_count == 1)
+		{
+		  if ((delimit_groups == DM_PRECEDE)
+		      || (delimit_groups == DM_SEPARATE
+			  && !first_delimiter))
+		    putc ('\n', ostream);
+		}
+	    }
+
 	  if (!match || mode == output_all_repeated)
 	    {
 	      writeline (prevline, ostream, match_count);
@@ -326,9 +378,12 @@ check_file (const char *infile, const char *outfile)
 int
 main (int argc, char **argv)
 {
-  int optc;
-  char *infile = "-", *outfile = "-";
+  int optc = 0;
+  int posixly_correct = (getenv ("POSIXLY_CORRECT") != NULL);
+  int nfiles = 0;
+  char const *file[2];
 
+  file[0] = file[1] = "-";
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -338,16 +393,50 @@ main (int argc, char **argv)
 
   skip_chars = 0;
   skip_fields = 0;
-  check_chars = 0;
+  check_chars = SIZE_MAX;
   mode = output_all;
   countmode = count_none;
+  delimit_groups = DM_NONE;
 
-  while ((optc = getopt_long (argc, argv, "0123456789cdDf:is:uw:", longopts,
-			      NULL)) != -1)
+  for (;;)
     {
-      switch (optc)
+      /* Parse an operand with leading "+" as a file after "--" was
+         seen; or if pedantic and a file was seen.  POSIX 1003.1-200x
+         d7 removes support for such operands, so when it becomes
+         official the code will need to be changed.  */
+
+      if (optc == -1
+	  || (posixly_correct && nfiles != 0)
+	  || ((optc = getopt_long (argc, argv,
+				   "-0123456789cdDf:is:uw:", longopts, NULL))
+	      == -1))
 	{
-	case 0:
+	  if (optind == argc)
+	    break;
+	  if (nfiles == 2)
+	    {
+	      error (0, 0, _("extra operand `%s'"), argv[optind]);
+	      usage (1);
+	    }
+	  file[nfiles++] = argv[optind++];
+	}
+      else switch (optc)
+	{
+	case 1:
+	  {
+	    unsigned long int size;
+	    if (optarg[0] == '+'
+		&& xstrtoul (optarg, NULL, 10, &size, "") == LONGINT_OK
+		&& size <= SIZE_MAX)
+	      skip_chars = size;
+	    else if (nfiles == 2)
+	      {
+		error (0, 0, _("extra operand `%s'"), optarg);
+		usage (1);
+	      }
+	    else
+	      file[nfiles++] = optarg;
+	  }
 	  break;
 
 	case '0':
@@ -360,7 +449,13 @@ main (int argc, char **argv)
 	case '7':
 	case '8':
 	case '9':
-	  skip_fields = skip_fields * 10 + optc - '0';
+	  {
+	    size_t s = skip_fields;
+	    skip_fields = s * 10 + optc - '0';
+	    if (SIZE_MAX / 10 < s || skip_fields < s)
+	      error (EXIT_FAILURE, 0, "%s",
+		     _("invalid number of fields to skip"));
+	  }
 	  break;
 
 	case 'c':
@@ -373,18 +468,17 @@ main (int argc, char **argv)
 
 	case 'D':
 	  mode = output_all_repeated;
+	  if (optarg == NULL)
+	    delimit_groups = DM_NONE;
+	  else
+	    delimit_groups = XARGMATCH ("--all-repeated", optarg,
+					delimit_method_string,
+					delimit_method_map);
 	  break;
 
 	case 'f':		/* Like '-#'. */
-	  {
-	    long int tmp_long;
-	    if (xstrtol (optarg, NULL, 10, &tmp_long, "") != LONGINT_OK
-		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (EXIT_FAILURE, 0,
-		     _("invalid number of fields to skip: `%s'"),
-		     optarg);
-	    skip_fields = (int) tmp_long;
-	  }
+	  skip_fields = size_opt (optarg,
+				  N_("invalid number of fields to skip"));
 	  break;
 
 	case 'i':
@@ -392,15 +486,8 @@ main (int argc, char **argv)
 	  break;
 
 	case 's':		/* Like '+#'. */
-	  {
-	    long int tmp_long;
-	    if (xstrtol (optarg, NULL, 10, &tmp_long, "") != LONGINT_OK
-		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (EXIT_FAILURE, 0,
-		     _("invalid number of bytes to skip: `%s'"),
-		     optarg);
-	    skip_chars = (int) tmp_long;
-	  }
+	  skip_chars = size_opt (optarg,
+				 N_("invalid number of bytes to skip"));
 	  break;
 
 	case 'u':
@@ -408,15 +495,8 @@ main (int argc, char **argv)
 	  break;
 
 	case 'w':
-	  {
-	    long int tmp_long;
-	    if (xstrtol (optarg, NULL, 10, &tmp_long, "") != LONGINT_OK
-		|| tmp_long <= 0 || tmp_long > INT_MAX)
-	      error (EXIT_FAILURE, 0,
-		     _("invalid number of bytes to compare: `%s'"),
-		     optarg);
-	    check_chars = (int) tmp_long;
-	  }
+	  check_chars = size_opt (optarg,
+				  N_("invalid number of bytes to compare"));
 	  break;
 
 	case_GETOPT_HELP_CHAR;
@@ -428,35 +508,6 @@ main (int argc, char **argv)
 	}
     }
 
-  if (argc - optind >= 2 && !STREQ (argv[optind - 1], "--"))
-    {
-      /* Interpret non-option arguments with leading `+' only
-	 if we haven't seen `--'.  */
-      while (optind < argc && argv[optind][0] == '+')
-	{
-	  char *opt_str = argv[optind++];
-	  long int tmp_long;
-	  if (xstrtol (opt_str, NULL, 10, &tmp_long, "") != LONGINT_OK
-	      || tmp_long <= 0 || tmp_long > INT_MAX)
-	    error (EXIT_FAILURE, 0,
-		   _("invalid number of bytes to skip: `%s'"),
-		   opt_str);
-	  skip_chars = (int) tmp_long;
-	}
-    }
-
-  if (optind < argc)
-    infile = argv[optind++];
-
-  if (optind < argc)
-    outfile = argv[optind++];
-
-  if (optind < argc)
-    {
-      error (0, 0, _("too many arguments"));
-      usage (1);
-    }
-
   if (countmode == count_occurrences && mode == output_all_repeated)
     {
       error (0, 0,
@@ -464,7 +515,7 @@ main (int argc, char **argv)
       usage (1);
     }
 
-  check_file (infile, outfile);
+  check_file (file[0], file[1]);
 
   exit (EXIT_SUCCESS);
 }
